Class: HeadMusic::Notation::ABC::ChordScanner::ChordNote

Inherits:
Data
  • Object
show all
Defined in:
lib/head_music/notation/abc/chord_scanner.rb

Overview

One note inside a bracket chord. Each note may carry its own length; the parser enforces that they are uniform and multiplies the shared inner length with any outer length on the token (ABC 2.1 sec. 4.17).

Instance Attribute Summary collapse

Instance Attribute Details

#accidentalObject (readonly)

Returns the value of attribute accidental

Returns:

  • (Object)

    the current value of accidental



13
14
15
# File 'lib/head_music/notation/abc/chord_scanner.rb', line 13

def accidental
  @accidental
end

#lengthObject (readonly)

Returns the value of attribute length

Returns:

  • (Object)

    the current value of length



13
14
15
# File 'lib/head_music/notation/abc/chord_scanner.rb', line 13

def length
  @length
end

#letterObject (readonly)

Returns the value of attribute letter

Returns:

  • (Object)

    the current value of letter



13
14
15
# File 'lib/head_music/notation/abc/chord_scanner.rb', line 13

def letter
  @letter
end

#octave_marksObject (readonly)

Returns the value of attribute octave_marks

Returns:

  • (Object)

    the current value of octave_marks



13
14
15
# File 'lib/head_music/notation/abc/chord_scanner.rb', line 13

def octave_marks
  @octave_marks
end