Class: HeadMusic::Style::Guidelines::FirstBarHalfNotes

Inherits:
FirstBarEntry show all
Defined in:
lib/head_music/style/guidelines/first_bar_half_notes.rb

Overview

Checks that the first bar contains half notes, with an optional half rest on beat one.

Constant Summary collapse

MESSAGE =
"Begin the first bar with half notes, or enter after a half rest."

Instance Method Summary collapse

Constructor Details

This class inherits a constructor from HeadMusic::Style::Annotation

Instance Method Details

#expected_rhythmic_valueObject (private)



10
11
12
# File 'lib/head_music/style/guidelines/first_bar_half_notes.rb', line 10

def expected_rhythmic_value
  HeadMusic::Rudiment::RhythmicValue.get(:half)
end