Class: HeadMusic::Style::Guidelines::FirstBarWholeNote

Inherits:
FirstBarEntry show all
Defined in:
lib/head_music/style/guidelines/first_bar_whole_note.rb

Overview

Checks that the first bar contains a whole note.

Constant Summary collapse

MESSAGE =
"Begin with a whole note in the first bar."

Instance Method Summary collapse

Constructor Details

This class inherits a constructor from HeadMusic::Style::Annotation

Instance Method Details

#expected_rhythmic_valueObject (private)



10
11
12
# File 'lib/head_music/style/guidelines/first_bar_whole_note.rb', line 10

def expected_rhythmic_value
  HeadMusic::Rudiment::RhythmicValue.get(:whole)
end