Class: HeadMusic::Style::Guidelines::ThirdSpeciesDissonanceTreatment

Inherits:
WeakBeatDissonanceTreatment show all
Defined in:
lib/head_music/style/guidelines/third_species_dissonance_treatment.rb

Overview

A counterpoint guideline for third-species dissonance treatment. Every dissonant note on beats 2, 3, or 4 must be treated as a passing tone, neighbor tone, nota cambiata, or double neighbor figure.

Constant Summary collapse

MESSAGE =
"Treat dissonances as passing tones, neighbor tones, cambiata, or double neighbor figures."

Instance Method Summary collapse

Constructor Details

This class inherits a constructor from HeadMusic::Style::Annotation

Instance Method Details

#recognized_figure?(note) ⇒ Boolean (private)

Returns:

  • (Boolean)


12
13
14
# File 'lib/head_music/style/guidelines/third_species_dissonance_treatment.rb', line 12

def recognized_figure?(note)
  super || neighbor_tone?(note) || cambiata_dissonance?(note) || double_neighbor_member?(note)
end